<?php

namespace B2buy\Support\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use B2buy\Support\Http\Requests\SupportRequest;
use B2buy\Support\Interfaces\SupportRepositoryInterface;
use B2buy\Support\Models\Support;

/**
 * Resource controller class for support.
 */
class SupportResourceController extends BaseController
{

    /**
     * Initialize support resource controller.
     *
     * @param type SupportRepositoryInterface $support
     *
     * @return null
     */
    public function __construct(SupportRepositoryInterface $support)
    {
        parent::__construct();
        $this->repository = $support;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\B2buy\Support\Repositories\Criteria\SupportResourceCriteria::class);
    }

    /**
     * Display a list of support.
     *
     * @return Response
     */
    public function index(SupportRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\B2buy\Support\Repositories\Presenter\SupportPresenter::class)
                ->$function();
        }

        $supports = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('support::support.names'))
            ->view('support::support.index', true)
            ->data(compact('supports', 'view'))
            ->output();
    }

    /**
     * Display support.
     *
     * @param Request $request
     * @param Model   $support
     *
     * @return Response
     */
    public function show(SupportRequest $request, Support $support)
    {

        if ($support->exists) {
            $view = 'support::support.show';
        } else {
            $view = 'support::support.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('support::support.name'))
            ->data(compact('support'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new support.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SupportRequest $request)
    {

        $support = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('support::support.name')) 
            ->view('support::support.create', true) 
            ->data(compact('support'))
            ->output();
    }

    /**
     * Create new support.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SupportRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $support                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('support::support.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('support/support/' . $support->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/support/support'))
                ->redirect();
        }

    }

    /**
     * Show support for editing.
     *
     * @param Request $request
     * @param Model   $support
     *
     * @return Response
     */
    public function edit(SupportRequest $request, Support $support)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('support::support.name'))
            ->view('support::support.edit', true)
            ->data(compact('support'))
            ->output();
    }

    /**
     * Update the support.
     *
     * @param Request $request
     * @param Model   $support
     *
     * @return Response
     */
    public function update(SupportRequest $request, Support $support)
    {
        try {
            $attributes = $request->all();

            $support->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('support::support.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('support/support/' . $support->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('support/support/' . $support->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the support.
     *
     * @param Model   $support
     *
     * @return Response
     */
    public function destroy(SupportRequest $request, Support $support)
    {
        try {

            $support->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('support::support.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('support/support/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('support/support/' . $support->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple support.
     *
     * @param Model   $support
     *
     * @return Response
     */
    public function delete(SupportRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('support::support.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('support/support'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/support/support'))
                ->redirect();
        }

    }

    /**
     * Restore deleted supports.
     *
     * @param Model   $support
     *
     * @return Response
     */
    public function restore(SupportRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('support::support.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/support/support'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/support/support/'))
                ->redirect();
        }

    }

}
